% Compute the L2 stress flux error of the individual gcells.
%
% function elerrs = flux_L2_error (self, geom, u, dT, nodal_stress)
%
% Input arguments
%     self     - finite element block
%     geom     - reference geometry field
%     u        - displacement field
%     dT       - temperature difference field, may be supplied as empty
%     nodal_stress   -  field of the nodal values of the stress. This would be computed as
%       nodal_stress = field_from_integration_points_spr(feb, geom, u, [], 'Cauchy', 1:4);
%       for a plane strain problem (as an example).  
% 
% Output argument
% Return an array of the element L2 errors of the flux.
%     elerrs= Array, number of columns equals the number of geometric
%     cells; for each finite element e the array component elerrs(e) holds the 
%     L2 norm of the error of the flux over the element 
%      
function elerrs = flux_L2_error (self, geom, u, dT, nodal_stress)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    % output
    dim=get(nodal_stress,'dim');
    errors=zeros(dim,length(gcells));
   % Now loop over all gcells in the block
    for i=1:length(gcells)
        conn = get(gcells(i), 'conn'); % connectivity
        x = gather(geom, conn, 'values', 'noreshape'); % coord
        U = gather(u, conn, 'values'); % displ
        if isempty(dT)
            dTs=zeros(nfens,1);
        else
            dTs=gather(dT, conn, 'values', 'noreshape');
        end
        nodsig = gather(nodal_stress, conn, 'values', 'noreshape'); % displ
         % Loop over all integration points
        for j=1:npts_per_gcell
            N = bfun(gcells(i),pc(j,:));
            Rm = material_directions(self,gcells(i),pc(j,:),x);
            detJ = Jacobian_volume(gcells(i),pc(j,:),x);
            B = Blmat(gcells(i), pc(j,:), x, Rm);
            context.strain = B*U;
            context.dT = dTs'*N;
            context.xyz =N'*x;
            [sig,ignore] = update(mat, matstates{i,j}, context);
            errors(:,i)=errors(:,i)+(sig-(N'*nodsig)').^2 * detJ * w(j);
        end
    end
    elerrs=sqrt(sum(errors,1));
end
