% Compute the  L2 heat flux error for the individual gcells.
%
% function elerrs = flux_L2_error (self, geom, Temp, nodal_flux)
%
% Input arguments
%     self     - finite element block
%     geom     - reference geometry field
%     u        - displacement field
%     dT       - temperature difference field, may be supplied as empty
%     nodal_flux   -  field of the nodal values of the heat flux. This would be computed as
%       nodal_flux = field_from_integration_points(feb, geom, u, [], 'flux', 1:2);
%       for a plane strain problem (as an example).  
% 
% Output argument
% Return an array of the element L2 errors of the flux.
%     elerrs= Array, number of columns equals the number of geometric
%     cells; for each finite element e the array component elerrs(e) holds the 
%     L2 norm of the error of the flux over the element 
%      
function elerrs = flux_L2_error (self, geom, Temp, nodal_flux)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    % output
    dim=get(nodal_flux,'dim');
    errors=zeros(dim,length(gcells));
   % Now loop over all gcells in the block
    for i=1:length(gcells)
        conn = get(gcells(i), 'conn'); % connectivity
        X = gather(geom, conn, 'values', 'noreshape'); % coord
        T = gather(Temp, conn, 'values'); % displ
        nodsig = gather(nodal_flux, conn, 'values', 'noreshape'); % displ
         % Loop over all integration points
        for j=1:npts_per_gcell
            N = bfun (gcells(i), pc(j,:));
            Rm=material_directions(self,gcells(i),pc(j,:),X);% do I need this?
            Nder = bfundpar (gcells(i), pc(j,:));
            NderX = bfundsp (gcells(i), Nder, X);
            context.ms=matstates{i,j};
            context.xyz=N'*X;
            context.gradtheta = grad(gcells(i), NderX, T);
            detJ = Jacobian_volume(gcells(i),pc(j,:),X);
            [sig,ignore] = update(mat, matstates{i,j}, context);
            errors(:,i)=errors(:,i)+(sig-(N'*nodsig)').^2 * detJ * w(j);
        end
    end
    elerrs=sqrt(sum(errors,1));
end
